/*
 * RandKeyCompAttacker.cc
 *
 *  Created on: May 18, 2015
 *      Author: Radim Ostadal
 */

#include "RandKeyCompAttacker.h"
#include "FindModule.h"
#include "utilfunctions.h"

Define_Module(RandKeyCompAttacker);

RandKeyCompAttacker::RandKeyCompAttacker()
    : BaseAttacker()
{}

void RandKeyCompAttacker::initialize(int stage) {
    BaseAttacker::initialize(stage);
}

void RandKeyCompAttacker::performCompromitation() {
    KeySet allKeys;
    int linksWithKey = 0;

    // Build the set of all neighbour keys from whole network and count them
    for (int i = 0; i < numNodes; i++) {
        cModule * nodeA = FindModule<>::findNetwork(this)->getSubmodule("node", i);
        Data * dataA = check_and_cast<Data *>(nodeA->getSubmodule("kms")->getSubmodule("data"));

        for (int j = i+1; j < numNodes; j++) {
            cModule * nodeB = FindModule<>::findNetwork(this)->getSubmodule("node", j);
            Data * dataB = check_and_cast<Data *>(nodeB->getSubmodule("kms")->getSubmodule("data"));

            if (areNodesNeighbours(nodeA, nodeB)) {

                if (existsSharedKeyBetween(nodeA->getIndex(), dataA, nodeB->getIndex(), dataB)) {
                    // Nodes A and B shares mutual key
                    linksWithKey++;

                    Key key = dataA->getKey(nodeB->getIndex());
                    allKeys.insert(key);
                }
            }
        }
    }

    // Compromise the relevant number of keys
    int keysToBeCaptured = attackerCompRate * linksWithKey;
    EV << "Number of keys to be compromised:" << keysToBeCaptured << endl;

    for(std::set<Key>::iterator it = allKeys.begin(); capturedKeys.size() < keysToBeCaptured; it++) {

        if (it == allKeys.end()) {
            it = allKeys.begin();
        }

        if (uniform(0, 1, 5) < attackerCompRate) {
            if ((capturedKeys.insert(*it)).second) {
                emit(attackerKeyStoredID, 1);
            }
        }
    }
}


RandKeyCompAttacker::~RandKeyCompAttacker() {

}
