#ifndef RAND_KEY_COMP_ATTACKER_H
#define RAND_KEY_COMP_ATTACKER_H

#include "BaseAttacker.h"

/**
 * @brief Implementation of Random key compromise attacker
 *
 * Attacker compromises link keys randomly. Only several keys might be
 * exfiltrated from particular node.
 *
 * @author Radim Ostadal
 **/
class MIXIM_API RandKeyCompAttacker : public BaseAttacker {

    private:
        /** @brief Copy constructor is not allowed */
        RandKeyCompAttacker(const RandKeyCompAttacker&);

        /** @brief Assignment operator is not allowed */
        RandKeyCompAttacker& operator=(const RandKeyCompAttacker&);

    public:
        RandKeyCompAttacker();
        virtual ~RandKeyCompAttacker();

        /** @brief Initialization of variables, activity planned using timers */
        virtual void initialize(int stage);

        /**
         * @brief Attacker performs initial compromise
         *
         * Firstly, the pool of all keys used in the network is prepared.
         * Secondly, the keys are randomly compromised up to the selected
         * threshold.
         **/
        void performCompromitation();
};

#endif
