#include "Debug.h"
#include "utilfunctions.h"
#include "FindModule.h"
#include "Data.h"
#include "BaseAttacker.h"

Define_Module(Debug);

Debug::Debug()
  : BaseModule()
  , numNodes(0)
{}

void Debug::initialize(int stage) {
    BaseModule::initialize(stage);

    if (stage == 0) {
        if (hasPar("numNodes")) numNodes = par("numNodes");

    } else if (stage == 1) {
        // Debugging predistributed keys
        //cMessage * debugPredistKeys = new cMessage("debugPredistKeys", PREDISTRIBUTION);
        //scheduleAt(0, debugPredistKeys);

        // Debugging neighbour keys
        //cMessage * debugNeighbourKeys = new cMessage("debugNeighbourKeys", NEIGHBOUR_KEY);
        //scheduleAt(2, debugNeighbourKeys);

        // Debugging exchanged nonces
        //cMessage * debugNeighbourNonces = new cMessage("debugNeighbourNonces", NEIGHBOUR_NONCES);
        //scheduleAt(1100, debugNeighbourNonces);

        // Debugging attacker compromitation
        //cMessage * debugAttacker = new cMessage("debugAttacker", ATTACKER);
        //scheduleAt(1100, debugAttacker);

        // Debugging node distances for hybrid protocols
        //cMessage * debugDistances = new cMessage("debugDistances", DISTANCES);
        //scheduleAt(1, debugDistances);
    }
}

void Debug::handleMessage(cMessage *msg) {
    if (msg->getKind() == PREDISTRIBUTION) {
        printPredistKeys();
    } else if (msg->getKind() == NEIGHBOUR_KEY) {
        printNeighbourKeys();
    } else if (msg->getKind() == NEIGHBOUR_NONCES) {
        printNeighbourNonces();
    } else if (msg->getKind() == ATTACKER) {
        printAttackerComp();
    } else if (msg->getKind() == DISTANCES) {
        printDistances();
    }

    delete msg;
}

void Debug::printPredistKeys() {
    for (int i = 0; i < numNodes; i++) {
        cModule * nodeA = getParentModule()->getSubmodule("node", i);
        Data * dataA = check_and_cast<Data*>(nodeA->getSubmodule("kms")->getSubmodule("data"));
        Keys predistributedKeys = dataA->getPredistributedKeys();

        for (Keys::iterator it = predistributedKeys.begin(); it != predistributedKeys.end(); it++) {
            EV << "Node " << i << " --> " << "Node " << it->first << endl;
            keyToHex(it->second, dataA->getKeyLength());
        }
    }
}

void Debug::printNeighbourKeys() {
    for (int i = 0; i < numNodes; i++) {
        cModule * nodeA = getParentModule()->getSubmodule("node", i);
        Data * dataA = check_and_cast<Data*>(nodeA->getSubmodule("kms")->getSubmodule("data"));
        Keys keys = dataA->getKeys();

        for (Keys::iterator it = keys.begin(); it != keys.end(); it++) {
            EV << "Node " << i << " --> " << "Node " << it->first << endl;
            keyToHex(it->second, dataA->getKeyLength());
        }
    }
}

void Debug::printNeighbourNonces() {
    for (int i = 0; i < numNodes; i++) {
        cModule * nodeA = getParentModule()->getSubmodule("node", i);
        Data * dataA = check_and_cast<Data*>(nodeA->getSubmodule("kms")->getSubmodule("data"));
        Nonces nonces = dataA->getNonces();

        for (Nonces::iterator it1 = nonces.begin(); it1 != nonces.end(); it1++) {
            Address neighbourAddress = it1->first;
            EV << "Node " << i << " --> " << "Neighbour " << neighbourAddress << endl;

            for (std::map<Hash, Nonce>::iterator it2 = nonces[neighbourAddress].begin(); it2 != nonces[neighbourAddress].end(); it2++) {
                EV << "Nonce hash: " << it2->first << " Nonce: ";
                keyToHex(it2->second, dataA->getKeyLength());
            }
        }
    }
}

void Debug::printAttackerComp() {
    BaseAttacker * attackerModule = check_and_cast<BaseAttacker *>(FindModule<>::findNetwork(this)->getSubmodule("attacker"));

    KeySet capturedKyes = attackerModule->getCapturedKeys();
    KeySet capturedNonces = attackerModule->getCapturedNonces();

    EV << "Attacker: Captured Kyes"  << endl;
    for (KeySet::iterator it = capturedKyes.begin(); it != capturedKyes.end(); it++) {
        EV << "     ";
        keyToHex(*it, 32);
    }

    EV << "Attacker: Captured Nonces"  << endl;
    for (KeySet::iterator it = capturedNonces.begin(); it != capturedNonces.end(); it++) {
        EV << "     ";
        keyToHex(*it, 32);
    }
}

void Debug::printDistances() {
    EV << numNodes << endl;
    for (int i = 0; i < numNodes; i++) {
        cModule * nodeA = getParentModule()->getSubmodule("node", i);
        Data * dataA = check_and_cast<Data*>(nodeA->getSubmodule("kms")->getSubmodule("data"));

        CommonNeighbourDistAll commonNeighbourDistAll = dataA->getCommonNeighbourDistAll();

        for (CommonNeighbourDistAll::iterator it1 = commonNeighbourDistAll.begin(); it1 != commonNeighbourDistAll.end(); it1++) {
            Address neighbourAddress = it1->first;
            EV << "Node " << i << " --> " << "Neighbour " << neighbourAddress << endl;

            for (std::map<Address, std::pair<double, double>>::iterator it2 = it1->second.begin(); it2 != it1->second.end(); it2++) {
                EV << "     Common neighbour: " << it2->first << " Distances: " << it2->second.first << "; " << it2->second.second << endl;
            }
        }
    }
}

Debug::~Debug() {

}
