#ifndef DEBUG_H
#define DEBUG_H

#include "BaseModule.h"

/**
 * @brief Centralised debugging features
 *
 * Debug module is responsible for centralised debugging features
 * like listing all keys on all nodes, all nonces on all nodes etc.
 *
 * @author Radim Ostadal
 **/
class Debug : public BaseModule {

    protected:
        /** @brief Number of legitimate nodes in the network */
        int numNodes;

        /** @breif Message types used for timers */
        enum selfMessages {
            PREDISTRIBUTION,
            NEIGHBOUR_KEY,
            NEIGHBOUR_NONCES,
            ATTACKER,
            NEIGHBOUR_COUNT,
            DISTANCES,
        };

    private:
        /** @brief Copy constructor is not allowed */
        Debug(const Debug&);

        /** @brief Assignment operator is not allowed */
        Debug& operator = (const Debug&);

    public:
        Debug();
        virtual ~Debug();

        /** @brief Initialization of variables, activity planned using timers */
        virtual void initialize(int stage);

        /** @brief Handle the self messages - start particular debugging activity */
        virtual void handleMessage(cMessage *msg);

        /** @brief Take all nodes and print all predistributed keys */
        void printPredistKeys();

        /** @brief Take all nodes and print all keys for every neighbour of that node */
        void printNeighbourKeys();

        /** @brief Take all nodes and print all nonces for every neighbour of that node */
        void printNeighbourNonces();

        /** @brief Print all keys and nonces compromised by an attacker */
        void printAttackerComp();

        /** @brief Print all nodes with their neighbours together with all common neighbours and respective distances */
        void printDistances();
};

#endif
