#ifndef BASE_KE_H
#define BASE_KE_H

#include "BaseApplLayer.h"
#include "Data.h"

/**
 * @brief Base class for link key establishment
 *
 * Key establishment defines a way how link keys are determined for neighbouring
 * nodes. Base class defines several help functions (e.g., setting the key).
 *
 * @author Radim Ostadal
 **/
class MIXIM_API BaseKE : public BaseApplLayer {
    protected:
        /** @brief Debug switch */
        bool debug;

        /** @brief Length of the network packet header (in bits) */
        int headerLength;

    private:
        /** @brief Copy constructor is not allowed */
        BaseKE(const BaseKE&);

        /** @brief Assignment operator is not allowed */
        BaseKE& operator=(const BaseKE&);

    public:
        BaseKE();
        virtual ~BaseKE();

        /** @brief Initialization of variables, activity planned using timers */
        virtual void initialize(int stage);

        /** @brief Return a pointer to Data structure of this node */
        Data * getDataModule() {return check_and_cast<Data *>(getParentModule()->getSubmodule("data"));}

        /** @brief Return the link key length */
        int getKeyLength() {return getDataModule()->getKeyLength();}

        /**
         * @brief Set the link key for particular neighbour
         *
         * @param addr Address of the neghbouring node
         * @param key Key to be set for my neighbour identified by addr
         * */
        void setKey(Address addr, Key key);
};

#endif
