#ifndef NEIGHBOUR_PREDISTRIBUTION_H
#define NEIGHBOUR_PREDISTRIBUTION_H

#include "BaseKE.h"

/**
 * @brief Class implementing the link key establishment
 *
 * The link keys are established based on predistributed material.
 * Only the keys for direct neighbours are used and copied to
 * Data structure Keys.
 *
 * @author Radim Ostadal
 **/
class MIXIM_API NeighbourPredistribution : public BaseKE {

    protected:
        /** @breif Message types used for timers */
        enum selfMessages {
            KE_INIT_KEYS,
        };

    private:
        /** @brief Copy constructor is not allowed */
        NeighbourPredistribution(const NeighbourPredistribution&);

        /** @brief Assignment operator is not allowed */
        NeighbourPredistribution& operator=(const NeighbourPredistribution&);

    public:
        NeighbourPredistribution();
        virtual ~NeighbourPredistribution();

        /** @brief Initialization of variables, activity planned using timers */
        virtual void initialize(int);

        /** @brief Function handling  messages generated by the timers */
        virtual void handleSelfMsg(cMessage* msg);

        /** @brief Establish link keys based on predistributed keys */
        virtual void predistNeighbourKeys();
};

#endif

