#include "PredistributionCentre.h"
#include "utilfunctions.h"

Define_Module(PredistributionCentre);

PredistributionCentre::PredistributionCentre()
  : BaseModule()
  , debug(false)
  , numNodes(0)
{}

void PredistributionCentre::initialize(int stage) {
    BaseModule::initialize(stage);

    if (stage == 0) {
		if(hasPar("debug")) debug = par("debug").boolValue();
		if (hasPar("numNodes")) numNodes = par("numNodes");

    } else if (stage == 1) {
	    if (!strcmp(par("predistributionType"), "PairwisePredistribution")) {

	        // Find the Data module to identify the key length before predistribution
	        cModule * node = getParentModule()->getSubmodule("node", 0);
	        Data * data = check_and_cast<Data*>(node->getSubmodule("kms")->getSubmodule("data"));

	        // Perform pairwise predistribution
	        pairwisePredistribution(data->getKeyLength());
	    }
	}
}

void PredistributionCentre::pairwisePredistribution(int keyLength) {
    Key keyA, keyB;

    for (int i = 0; i < numNodes; i++) {
        cModule * nodeA = getParentModule()->getSubmodule("node", i);
        Data * dataA = check_and_cast<Data*>(nodeA->getSubmodule("kms")->getSubmodule("data"));

        for (int j = i+1; j < numNodes; j++) {
            cModule * nodeB = getParentModule()->getSubmodule("node", j);
            Data * dataB = check_and_cast<Data*>(nodeB->getSubmodule("kms")->getSubmodule("data"));

            keyA = new unsigned char [keyLength];
            keyB = new unsigned char [keyLength];

            rndArray(keyA, keyLength);
            memcpy(keyB, keyA, keyLength);

            dataA->setPredistributedKey(j,keyA);
            dataB->setPredistributedKey(i,keyB);
        }
    }
}

PredistributionCentre::~PredistributionCentre() {

}
