#ifndef PREDISTRIBUTION_CENTRE_H
#define PREDISTRIBUTION_CENTRE_H

#include "BaseModule.h"

/**
 * @brief Class for predistribution centre
 *
 * Predistribution centre distributes all keying material for legal nodes
 * during the initialization phase. Only the pairwise predistribution is
 * implemented for now.
 *
 * @author Radim Ostadal
 **/
class MIXIM_API PredistributionCentre : public BaseModule {

    protected:
        /** @brief Debug switch */
        bool debug;

        /** @brief Number of legitimate nodes in the network */
        int numNodes;

    private:
        /** @brief Copy constructor is not allowed */
        PredistributionCentre(const PredistributionCentre&);

        /** @brief Assignment operator is not allowed */
        PredistributionCentre& operator = (const PredistributionCentre&);

    public:
        PredistributionCentre();
        virtual ~PredistributionCentre();

        /** @brief Initialization of variables, activity planned using timers */
        virtual void initialize(int stage);

        /**
         * @brief All pairs of legitimate nodes are provided with unique key;
         * keys are stored in dedicated array "predistributedKeys" in Data module
         **/
        void pairwisePredistribution(int keyLength);
    };

#endif
