#include "BaseSA.h"

Define_Module(BaseSA);

BaseSA::BaseSA()
    : BaseApplLayer()
    , debug(false)
    , headerLength(0)
    , noncePeriod(0)
    , amplifRepetitions(0)
    , amplifPeriod(0)
    , protocolType(0)
    , dataModule()
    , delayTimer()
{}

void BaseSA::initialize (int stage) {
    BaseApplLayer::initialize(stage);

    if (stage == 0) {
        if (hasPar("debug")) debug = par("debug").boolValue();
        if (hasPar("headerLength")) headerLength = par("headerLength");
        if (hasPar("noncePeriod")) noncePeriod = par("noncePeriod");
        if (hasPar("amplifRepetitions")) amplifRepetitions = par("amplifRepetitions");
        if (hasPar("amplifPeriod")) amplifPeriod = par("amplifPeriod");

    } else if (stage == 1) {
        delayTimer = new cMessage("Starting to send nonce pkts", EXECUTE_PROTOCOL);
        delayTimer->setSchedulingPriority(2);
        scheduleAt(0, delayTimer);

        dataModule = check_and_cast<Data *>(getParentModule()->getSubmodule("data"));
    }
}

void BaseSA::handleSelfMsg(cMessage* msg) {

    if (msg->getKind() == EXECUTE_PROTOCOL) {

        executeProtocol();

        amplifRepetitions--;

        if (amplifRepetitions > 0) {
            delayTimer = new cMessage("Starting to send nonce pkt", EXECUTE_PROTOCOL);
            scheduleAt(simTime() + amplifPeriod, delayTimer);
        }

    } else if (msg->getKind() == INSERT_NONCE) {
        NoncePkt *pkt  = check_and_cast<NoncePkt*>(msg);

        int keyLength = dataModule->getKeyLength();

        // Get nonce from the packet
        Nonce nonce = new unsigned char[keyLength];
        for (int i = 0; i < keyLength; i++) {
            nonce[i] = pkt->getNonce(i);
        }

        // Insert the nonce to the nonces array in Data modul
        dataModule->insertNonce(pkt->getNeighbourNode(), nonce);
    }

    delete msg;
}

void BaseSA::executeProtocol() {

}

void BaseSA::processNoncePkt(cMessage* msg) {

    NoncePkt *pkt  = check_and_cast<NoncePkt*>(msg);

    if (pkt->getDstNode() == myApplAddr()) {

        int keyLength = dataModule->getKeyLength();

        // Get nonce from the packet
        Nonce nonce = new unsigned char[keyLength];
        for (int i = 0; i < keyLength; i++) {
            nonce[i] = pkt->getNonce(i);
        }

        // Insert the nonce to the nonces array in Data modul
        dataModule->insertNonce(pkt->getNeighbourNode(), nonce);

        emit(amplifNonceReceivedTotalID, 1);

        if ((pkt->getInterNode1() != -1) || (pkt->getInterNode2() != -1)) {
            emit(amplifNonceReceivedMultihoplID, 1);
        } else {
            emit(amplifNonceReceivedDirectID, 1);
        }

        delete pkt;
    } else if (pkt->getInterNode2() == myApplAddr()) {
        // Resent the packet to final destination node
        Address nextHopNode = pkt->getDstNode();

        // Remove original control info for network layer
        pkt->removeControlInfo();

        // Check that the next hop node is my neighbour
        if (!dataModule->isMyNeighbour(nextHopNode)) {
            emit(amplifNonceDiscardInter2ID, 1);

            delete pkt;
            return;
        }

        // Set control info for network layer
        NetwControlInfo::setControlInfo(pkt, nextHopNode);

        // Sending packet with small random delay
        dataModule->passOutNoncePkt(pkt, uniform(0,0.1));

        emit(amplifNonceReSentInter2ID, 1);

    } else if (pkt->getInterNode1() == myApplAddr()) {
        // Resent the packet to InterNode2 node
        Address nextHopNode = pkt->getInterNode2();

        // Remove original control info for network layer
        pkt->removeControlInfo();

        // Check that the next hop node is my neighbour
        if (!dataModule->isMyNeighbour(nextHopNode)) {
            emit(amplifNonceDiscardInter1ID, 1);

            delete pkt;
            return;
        }

        // Set control info for network layer
        NetwControlInfo::setControlInfo(pkt, nextHopNode);

        // Sending packet with small random delay
        dataModule->passOutNoncePkt(pkt, uniform(0,0.1));

        emit(amplifNonceReSentInter1ID, 1);
    }
}

BaseSA::~BaseSA() {

}
