#include "NodeOriented.h"

Define_Module(NodeOriented);

NodeOriented::NodeOriented()
    : BaseSA()
{}

void NodeOriented::initialize(int stage) {
    BaseSA::initialize(stage);

    if (stage == 0) {
        if (hasPar("protocolType")) {
            if (!strcmp(par("protocolType"), "PULL")) {protocolType = PULL_NONCE;}
            if (!strcmp(par("protocolType"), "PUSH")) {protocolType = PUSH_NONCE;}
            if (!strcmp(par("protocolType"), "MULTIPULL")) {protocolType = MULTI_PULL_NONCE;}
            if (!strcmp(par("protocolType"), "MULTIPUSH")) {protocolType = MULTI_PUSH_NONCE;}
            if (!strcmp(par("protocolType"), "NO_BEST_ORIG")) {protocolType = NO_BEST_ORIG_NONCE;}
            if (!strcmp(par("protocolType"), "NO_BEST_NEW")) {protocolType = NO_BEST_NEW_NONCE;}
        }

    } else if (stage == 1) {

    }
}

void NodeOriented::executeProtocol() {
    if (protocolType == PULL_NONCE) {
        performPull();
    } else if (protocolType == PUSH_NONCE) {
        performPush();
    } else if (protocolType == MULTI_PULL_NONCE) {
        performMultiPull();
    } else if (protocolType == MULTI_PUSH_NONCE) {
        performMultiPush();
    } else if (protocolType == NO_BEST_ORIG_NONCE) {
        performPush();
        performMultiPull();
    } else if (protocolType == NO_BEST_NEW_NONCE) {
        performPush();
        performMultiPush();
    }
}

void NodeOriented::performPull() {

    Keys keys = dataModule->getKeys();
    Address neighbourA, neighbourB;

    // Iterate through all pairs of neighbours and provide them with nonces
    for (Keys::iterator it1 = keys.begin(); it1 != keys.end(); it1++) {
        neighbourA = it1->first;

        for (Keys::iterator it2 = it1; it2 != keys.end(); it2++) {
            neighbourB = it2->first;

            // Skip the case where both it1 and it2 are the same node
            if (neighbourA == neighbourB) {continue;}

            NoncePkt *pktA = new NoncePkt("Pull nonce", Data::SA_NONCE_PKT);
            NoncePkt *pktB;

            // Generate random nonce
            int keyLength = dataModule->getKeyLength();
            Nonce nonce = new unsigned char [keyLength];
            rndArray(nonce, keyLength);

            emit(amplifNonceGeneratedID, 1);

            // Set nonce to packet
            pktA->setNonceArraySize(keyLength);
            for (int i = 0; i < keyLength; i++) {
                pktA->setNonce(i, nonce[i]);
            }

            // Set the source node
            pktA->setSrcNode(myApplAddr());

            // All common parameters are set, duplicate the packet
            pktB = pktA->dup();

            // Set the destination nodes and their neighbour
            pktA->setDstNode(neighbourA);
            pktA->setNeighbourNode(neighbourB);

            pktB->setDstNode(neighbourB);
            pktB->setNeighbourNode(neighbourA);

            // Send the packet
            NetwControlInfo::setControlInfo(pktA, neighbourA);
            NetwControlInfo::setControlInfo(pktB, neighbourB);

            // Generate the delay for the packets
            double pktDelay = uniform(0, noncePeriod);

            dataModule->passOutNoncePkt(pktA, pktDelay + uniform(0, 0.1));
            emit(amplifNonceSentID, 1);
            dataModule->passOutNoncePkt(pktB, pktDelay + uniform(0, 0.1));
            emit(amplifNonceSentID, 1);
        }
    }
}

void NodeOriented::performPush() {
    Keys keys = dataModule->getKeys();
    Address dstNode, interNode2;

    // Iterate through all my neighbours (take destination node)
    for (Keys::iterator it1 = keys.begin(); it1 != keys.end(); it1++) {
        dstNode = it1->first;

        // Iterate through all my neighbours (take intermediate node)
        for (Keys::iterator it2 = keys.begin(); it2 != keys.end(); it2++) {
            interNode2 = it2->first;

            // Skip the case where both it1 and it2 are the same node
            if (dstNode == interNode2) {continue;}

            NoncePkt *pkt = new NoncePkt("Push nonce", Data::SA_NONCE_PKT);
            NoncePkt *selfPkt = new NoncePkt("Self push nonce", INSERT_NONCE);

            // Generate random nonce
            int keyLength = dataModule->getKeyLength();
            Nonce nonce = new unsigned char [keyLength];
            rndArray(nonce, keyLength);

            emit(amplifNonceGeneratedID, 1);

            // Set nonce to packet
            pkt->setNonceArraySize(keyLength);
            selfPkt->setNonceArraySize(keyLength);
            for (int i = 0; i < keyLength; i++) {
                pkt->setNonce(i, nonce[i]);
                selfPkt->setNonce(i, nonce[i]);
            }

            // Set the source node, destination node, neighbour node and intermediate node
            pkt->setDstNode(dstNode);
            pkt->setSrcNode(myApplAddr());
            pkt->setNeighbourNode(myApplAddr());
            pkt->setInterNode2(interNode2);

            // Generate the delay for the packets
            double pktDelay = uniform(0, noncePeriod);

            // Set the neighbouring node for self msg (my neighbour for which the nonce was generated)
            selfPkt->setNeighbourNode(dstNode);

            // Schedule the self packet
            scheduleAt(simTime() + pktDelay, selfPkt);

            emit(amplifNonceSavedID, 1);

            // Send the packet
            NetwControlInfo::setControlInfo(pkt, interNode2);

            dataModule->passOutNoncePkt(pkt, pktDelay);
            emit(amplifNonceSentID, 1);
        }
    }
}


void NodeOriented::performMultiPull() {

    Keys keys = dataModule->getKeys();
    Address neighbourA, neighbourB, interNode2;

    // Iterate through all pairs of neighbours and provide them with nonces
    for (Keys::iterator it1 = keys.begin(); it1 != keys.end(); it1++) {
        neighbourA = it1->first;

        for (Keys::iterator it2 = keys.begin(); it2 != keys.end(); it2++) {
            neighbourB = it2->first;

            // Skip the case where both it1 and it2 are the same node
            if (neighbourA == neighbourB) {continue;}

            for (Keys::iterator it3 = keys.begin(); it3 != keys.end(); it3++) {
                interNode2 = it3->first;

                // Skip the case where any of it1, it2 and it3 are the same node
                if (interNode2 == neighbourA) {continue;}
                if (interNode2 == neighbourB) {continue;}

                NoncePkt *pktA = new NoncePkt("Multi pull nonce", Data::SA_NONCE_PKT);
                NoncePkt *pktB;

                // Generate random nonce
                int keyLength = dataModule->getKeyLength();
                Nonce nonce = new unsigned char [keyLength];
                rndArray(nonce, keyLength);

                emit(amplifNonceGeneratedID, 1);

                // Set nonce to packet
                pktA->setNonceArraySize(keyLength);
                for (int i = 0; i < keyLength; i++) {
                    pktA->setNonce(i, nonce[i]);
                }

                // Set the source node
                pktA->setSrcNode(myApplAddr());

                // All common parameters are set, duplicate the packet
                pktB = pktA->dup();

                // Set the destination nodes, their neighbour and intermediate node
                pktA->setDstNode(neighbourA);
                pktA->setNeighbourNode(neighbourB);

                pktB->setDstNode(neighbourB);
                pktB->setNeighbourNode(neighbourA);
                pktB->setInterNode2(interNode2);

                // Send the packet
                NetwControlInfo::setControlInfo(pktA, neighbourA);
                NetwControlInfo::setControlInfo(pktB, interNode2);

                // Generate the delay for the packets
                double pktDelay = uniform(0, noncePeriod);

                dataModule->passOutNoncePkt(pktA, pktDelay + uniform(0, 0.1));
                emit(amplifNonceSentID, 1);
                dataModule->passOutNoncePkt(pktB, pktDelay + uniform(0, 0.1));
                emit(amplifNonceSentID, 1);
            }
        }
    }
}

void NodeOriented::performMultiPush() {
    Keys keys = dataModule->getKeys();
    Address dstNode, interNode1, interNode2;

    // Iterate through all my neighbours (take destination node)
    for (Keys::iterator it1 = keys.begin(); it1 != keys.end(); it1++) {
        dstNode = it1->first;

        // Iterate through all my neighbours (take first intermediate node)
        for (Keys::iterator it2 = keys.begin(); it2 != keys.end(); it2++) {
            interNode1 = it2->first;

            // Skip the case where both it1 and it2 are the same node
            if (interNode1 == dstNode) {continue;}

            for (Keys::iterator it3 = keys.begin(); it3 != keys.end(); it3++) {
                interNode2 = it3->first;

                // Skip the case where any of it1, it2 and it3 are the same node
                if (interNode2 == dstNode) {continue;}
                if (interNode2 == interNode1) {continue;}

                NoncePkt *pkt = new NoncePkt("Multi push nonce", Data::SA_NONCE_PKT);
                NoncePkt *selfPkt = new NoncePkt("Self push nonce", INSERT_NONCE);

                // Generate random nonce
                int keyLength = dataModule->getKeyLength();
                Nonce nonce = new unsigned char [keyLength];
                rndArray(nonce, keyLength);

                emit(amplifNonceGeneratedID, 1);

                // Set nonce to packet
                pkt->setNonceArraySize(keyLength);
                selfPkt->setNonceArraySize(keyLength);
                for (int i = 0; i < keyLength; i++) {
                    pkt->setNonce(i, nonce[i]);
                    selfPkt->setNonce(i, nonce[i]);
                }

                // Set the source node, destination node, neighbour node and intermediate nodes
                pkt->setDstNode(dstNode);
                pkt->setSrcNode(myApplAddr());
                pkt->setNeighbourNode(myApplAddr());
                pkt->setInterNode1(interNode1);
                pkt->setInterNode2(interNode2);

                // Generate the delay for the packets
                double pktDelay = uniform(0, noncePeriod);

                // Set the neighbouring node for self msg (my neighbour for which the nonce was generated)
                selfPkt->setNeighbourNode(dstNode);

                // Plan the self packet
                scheduleAt(simTime() + pktDelay, selfPkt);

                emit(amplifNonceSavedID, 1);

                // Send the packet
                NetwControlInfo::setControlInfo(pkt, interNode1);

                dataModule->passOutNoncePkt(pkt, pktDelay);
                emit(amplifNonceSentID, 1);
            }
        }
    }
}

NodeOriented::~NodeOriented() {

}
