#include "SwitchLayer.h"
#include "ApplPkt_m.h"
#include "FindModule.h"
#include "NoncePkt_m.h"
#include "NetwControlInfo.h"

Define_Module(SwitchLayer);

SwitchLayer::SwitchLayer()
    : BaseModule()
    , debug(false)
    , upperAppLayerIn(-1)
    , upperAppLayerOut(-1)
    , upperAppControlIn(-1)
    , upperAppControlOut(-1)
    , upperKMSLayerIn(-1)
    , upperKMSLayerOut(-1)
    , upperKMSControlIn(-1)
    , upperKMSControlOut(-1)
    , lowerLayerIn(-1)
    , lowerLayerOut(-1)
    , lowerControlIn(-1)
    , lowerControlOut(-1)
    , dataModule()
    , attackerModule()
    , mobilityModule()
    , nodeID(0)
{}

void SwitchLayer::initialize(int stage) {
    BaseModule::initialize(stage);

    if (stage == 0) {
        if (hasPar("debug")) debug = par("debug").boolValue();

        upperAppLayerIn = findGate("upperAppLayerIn");
        upperAppLayerOut = findGate("upperAppLayerOut");
        upperAppControlIn = findGate("upperAppControlIn");
        upperAppControlOut = findGate("upperAppControlOut");

        upperKMSLayerIn = findGate("upperKMSLayerIn");
        upperKMSLayerOut = findGate("upperKMSLayerOut");
        upperKMSControlIn = findGate("upperKMSControlIn");
        upperKMSControlOut = findGate("upperKMSControlOut");

        lowerLayerIn = findGate("lowerLayerIn");
        lowerLayerOut = findGate("lowerLayerOut");
        lowerControlIn = findGate("lowerControlIn");
        lowerControlOut = findGate("lowerControlOut");

    } else if (stage == 1) {
        // Get data module
        dataModule = check_and_cast<Data *>(getParentModule()->getSubmodule("kms")->getSubmodule("data"));
        // Get attacker module
        attackerModule = check_and_cast<BaseAttacker *>(FindModule<>::findNetwork(this)->getSubmodule("attacker"));
        // Get mobility module
        mobilityModule = check_and_cast<StaticMobility*>(getParentModule()->getSubmodule("mobility"));

        nodeID = getNode()->getIndex();
    }
}

void SwitchLayer::handleMessage(cMessage* msg) {

    if (msg->getArrivalGateId() == upperAppLayerIn || msg->getArrivalGateId() == upperKMSLayerIn) {
        handleUpperMsg(msg);
    } else if (msg->getArrivalGateId() == upperAppControlIn || msg->getArrivalGateId() == upperKMSControlIn) {
        handleUpperControl(msg);
    } else if (msg->getArrivalGateId() == lowerLayerIn) {
        handleLowerMsg(msg);
    } else if (msg->getArrivalGateId() == lowerControlIn) {
        handleLowerControl(msg);
    }
}

void SwitchLayer::handleUpperMsg(cMessage* msg) {
    if (msg->getKind() == Data::SA_NONCE_PKT) {
        NoncePkt *pkt  = check_and_cast<NoncePkt*>(msg);

        Address nextHopNode = NetwControlInfo::getAddressFromControlInfo(pkt->getControlInfo());

        // Get nonce from the packet
        Nonce nonce = new unsigned char[32];
        for (int i = 0; i < 32; i++) {
            nonce[i] = pkt->getNonce(i);
        }

        Coord nodePosition = mobilityModule->getCurrentPosition();

        // Perform check against attacker
        attackerModule->attackerNonceSentCheck(nodeID, dataModule->getKey(nextHopNode), nonce, nodePosition);
    }

    send(msg, lowerLayerOut);
}

void SwitchLayer::handleLowerMsg(cMessage* msg) {

    if (msg->getKind() == Data::SA_NONCE_PKT) {
        NoncePkt *pkt  = check_and_cast<NoncePkt*>(msg);

        // Get nonce from the packet
        Nonce nonce = new unsigned char[32];
        for (int i = 0; i < 32; i++) {
            nonce[i] = pkt->getNonce(i);
        }

        // Perform check against attacker
        attackerModule->attackerNonceReceivedCheck(nodeID, nonce);
    }

    if (typeid(*msg) == typeid(ApplPkt)) {
        send(msg, upperAppLayerOut);
    } else {
        send(msg, upperKMSLayerOut);
    }
}

void SwitchLayer::handleUpperControl(cMessage* msg) {
    delete msg;
}

void SwitchLayer::handleLowerControl(cMessage* msg) {
    delete msg;
}

SwitchLayer::~SwitchLayer() {

}
