#ifndef SWITCH_LAYER_H
#define SWITCH_LAYER_H

#include "BaseModule.h"
#include "Data.h"
#include "BaseAttacker.h"
#include "StaticMobility.h"

/**
 * @brief Class for switch layer - interface among the application layer,
 * key management scheme layer and network layer
 *
 * The switch is responsible to direct the traffic either to application
 * layer or key management scheme layer. It is also responsible to do all
 * checks against the attacker.
 *
 * @author Radim Ostadal
 **/
class MIXIM_API SwitchLayer : public BaseModule {

    protected:
        /** @brief Debug switch */
        bool debug;

        /** @brief Gate from application layer (Gate ID) */
        int upperAppLayerIn;

        /** @brief Gate to application layer (Gate ID) */
        int upperAppLayerOut;

        /** @brief Control from application layer (Gate ID) */
        int upperAppControlIn;

        /** @brief Control to application layer (Gate ID) */
        int upperAppControlOut;

        /** @brief Gate from kms layer (Gate ID) */
        int upperKMSLayerIn;

        /** @brief Gate to kms layer (Gate ID) */
        int upperKMSLayerOut;

        /** @brief Control from kms layer (Gate ID) */
        int upperKMSControlIn;

        /** @brief Control to kms layer (Gate ID) */
        int upperKMSControlOut;

        /** @brief Gate from network layer (Gate ID) */
        int lowerLayerIn;

        /** @brief Gate to network layer (Gate ID) */
        int lowerLayerOut;

        /** @brief Control from network layer (Gate ID) */
        int lowerControlIn;

        /** @brief Control to network layer (Gate ID) */
        int lowerControlOut;

        /** @brief Pointer to data module */
        Data* dataModule;

        /** @brief Pointer to attacker module */
        BaseAttacker* attackerModule;

        /** @brief Pointer to mobility modul */
        StaticMobility* mobilityModule;

        /** @brief Node identification number (node address) */
        int nodeID;

    private:
        /** @brief Copy constructor is not allowed */
        SwitchLayer(const SwitchLayer&);

        /** @brief Assignment operator is not allowed */
        SwitchLayer& operator=(const SwitchLayer&);

    public:
        SwitchLayer();
        virtual ~SwitchLayer();

        /** @brief Initialization of the module and some variables */
        virtual void initialize(int stage);

        /** @brief Called every time a message arrives */
        virtual void handleMessage(cMessage* msg);

        /** @brief Handle messages from upper layer; if the message is nonce packet, the check against the attacker is performed */
        virtual void handleUpperMsg(cMessage *msg);

        /** @brief Handle messages from lower layer; if the message is nonce packet, the check against the attacker is performed */
        virtual void handleLowerMsg(cMessage *msg);

        /** @brief Handle control messages from upper layer */
        virtual void handleUpperControl(cMessage *msg);

        /** @brief Handle control messages from lower layer */
        virtual void handleLowerControl(cMessage *msg);
};

#endif
