#include "SimHelper.h"
#include <fstream>
#include <FindModule.h>
#include "StaticMobility.h"
#include "utilfunctions.h"
#include "Data.h"

using namespace std;

Define_Module(SimHelper);

SimHelper::SimHelper()
    : BaseModule()
    , debug(false)
    , numNodes(0)
    , playgroundSizeX(0)
    , playgroundSizeY(0)
    , playgroundSizeZ(0)
{}

void SimHelper::initialize(int stage){
	BaseModule::initialize(stage);

	if (stage == 0) {
	    if (hasPar("debug")) debug = par("debug").boolValue();
        if (hasPar("numNodes")) numNodes = par("numNodes");
        if (hasPar("playgroundSizeX")) playgroundSizeX = par("playgroundSizeX");
        if (hasPar("playgroundSizeY")) playgroundSizeY = par("playgroundSizeY");
        if (hasPar("playgroundSizeZ")) playgroundSizeZ = par("playgroundSizeZ");

	} else if (stage == 1) {

	    //initMobility();
		initMobilityRandomPositions();

		cMessage *initNeighbourDistances = new cMessage("initNeighbourDistances", INIT_NEIGHBOUR_DIST);
		scheduleAt(0, initNeighbourDistances);
	}
}

void SimHelper::handleMessage(cMessage *msg) {
    if (msg->getKind() == INIT_NEIGHBOUR_DIST) {
        initNeighbourDistances();
    }

    delete msg;
}

// This function is used for "RSSI" computation - necessary for Hybrid Design protocols
void SimHelper::initNeighbourDistances() {

    Coord coordA, coordB, coordC;

    for (int i = 0; i < numNodes; i++) {
        cModule * nodeA = getParentModule()->getSubmodule("node", i);
        Data * dataA = check_and_cast<Data*>(nodeA->getSubmodule("kms")->getSubmodule("data"));
        StaticMobility * mobilityA = check_and_cast<StaticMobility*>(nodeA->getSubmodule("mobility"));

        for (int j = i+1; j < numNodes; j++) {
            cModule * nodeB = getParentModule()->getSubmodule("node", j);
            Data * dataB = check_and_cast<Data*>(nodeB->getSubmodule("kms")->getSubmodule("data"));
            StaticMobility * mobilityB = check_and_cast<StaticMobility*>(nodeB->getSubmodule("mobility"));

            for (int k = j+1; k < numNodes; k++) {
                cModule * nodeC = getParentModule()->getSubmodule("node", k);
                Data * dataC = check_and_cast<Data*>(nodeC->getSubmodule("kms")->getSubmodule("data"));
                StaticMobility * mobilityC = check_and_cast<StaticMobility*>(nodeC->getSubmodule("mobility"));

                // All three nodes have to be neighbours
                if (areNodesNeighbours(nodeA, nodeB) && areNodesNeighbours(nodeA, nodeC) && areNodesNeighbours(nodeB, nodeC)) {

                    coordA = mobilityA->getCurrentPosition();
                    coordB = mobilityB->getCurrentPosition();
                    coordC = mobilityC->getCurrentPosition();

                    // Fill the distance structure commonNeighbourDist for each other
                    dataA->setCommonNeighbourDist(j, k, coordA.distance(coordC), coordB.distance(coordC));
                    dataA->setCommonNeighbourDist(k, j, coordA.distance(coordB), coordC.distance(coordB));

                    dataB->setCommonNeighbourDist(i, k, coordB.distance(coordC), coordA.distance(coordC));
                    dataB->setCommonNeighbourDist(k, i, coordB.distance(coordA), coordC.distance(coordA));

                    dataC->setCommonNeighbourDist(i, j, coordC.distance(coordB), coordA.distance(coordB));
                    dataC->setCommonNeighbourDist(j, i, coordC.distance(coordA), coordB.distance(coordA));
                }
            }
        }
    }
}

void SimHelper::initMobilityRandomPositions() {
    double x,y,z;

    for (int i = 0; i < numNodes; i++) {
        cModule* node = getParentModule()->getSubmodule("node", i);
        StaticMobility* mobility = FindModule<StaticMobility*>::findSubModule(node);

        x = uniform(0, playgroundSizeX);
        y = uniform(0, playgroundSizeY);
        z = uniform(0, playgroundSizeZ);

        mobility->setPosition(Coord(x, y, z));
    }
}

void SimHelper::initMobility() {
	const char *mobilityFile = par("mobilityFile");
	unsigned int countLoaded = 0;

	cModule* node;
	StaticMobility* mobility;

	ifstream ifstr (mobilityFile);
	if (ifstr.is_open()){
		string line;
		int id;
		double x, y, z;

		while (ifstr.good()){
			ifstr >> line;

			unsigned int tmp = line.find(";", 0);
			if (tmp == string::npos) continue;
			id = atoi(line.substr(0,tmp).c_str());

			line = line.substr(tmp+1,line.length());
			if ((tmp = line.find(";", 0)) == string::npos) continue;
			x = atof(line.substr(0,tmp).c_str());

			line = line.substr(tmp+1,line.length());
			if ((tmp = line.find(";", 0)) == string::npos) continue;
			y = atof(line.substr(0,tmp).c_str());

			line = line.substr(tmp+1,line.length());
			z = atof(line.c_str());

			node = getParentModule()->getSubmodule("node", id);
			if (!node) continue;

			mobility = FindModule<StaticMobility*>::findSubModule(node);
			if (!mobility) continue;
			EV << "Current position of node " << id << "is: " << mobility->getCurrentPosition() << endl;

			mobility->par("initialX") = x;
			mobility->par("initialY") = y;
			mobility->par("initialZ") = z;

			mobility->setPosition(Coord(x, y, z));
			countLoaded++;
		}
		ifstr.close();
	}
}

SimHelper::~SimHelper() {

}



