#ifndef SIM_HELPER_H
#define SIM_HELPER_H

#include <BaseModule.h>

/**
 * @brief Class for simulation helping functions (e.g., initialization
 * of legitimate node's position from configuration file)
 *
 * @author Radim Ostadal
 **/
class SimHelper : public BaseModule {

    protected:
        /** @brief Debug switch */
        bool debug;

        /** @brief Number of legitimate nodes in the network */
        int numNodes;

        /** @brief X size of the area the nodes are in (in meters) */
        double playgroundSizeX;

        /** @brief Y size of the area the nodes are in (in meters) */
        double playgroundSizeY;

        /** @brief Z size of the area the nodes are in (in meters) */
        double playgroundSizeZ;

        /** @breif Message types used for timers */
        enum selfMessages {
            INIT_NEIGHBOUR_DIST,
        };

    private:
        /** @brief Copy constructor is not allowed */
        SimHelper(const SimHelper&);

        /** @brief Assignment operator is not allowed */
        SimHelper& operator = (const SimHelper&);

    public:
        SimHelper();
        virtual ~SimHelper();

        /** @brief Initialization of variables, initialization of legitimate nodes position */
        virtual void initialize(int stage);

        /** @brief Handle the self messages - start the computation of distances (necessary for hybrid designed protocols) */
        void handleMessage(cMessage *msg);

        /** @brief Initiate the legitimate nodes position from file */
        void initMobility();

        /** @brief Initiate the legitimate nodes position randomly */
        void initMobilityRandomPositions();

        /** @brief Every node is provided with mutual neighbors for every its neighbour together with respective distances */
        void initNeighbourDistances();
};

#endif
