#ifndef KMSDEFS_H_
#define KMSDEFS_H_

#include "SimpleAddress.h"

/** @brief Type for the link key */
typedef unsigned char * Key;

/** @brief Type for the nonce */
typedef unsigned char * Nonce;

/** @brief Type for the nonce hash */
typedef unsigned int Hash;

/** @brief Define how to compare two keys */
struct key_compare {
    bool operator()(const Key a, const Key b) const {
        return memcmp(a, b, 32) < 0;
    }
};

/** @brief Set of kyes */
typedef std::set<Key, key_compare> KeySet;

/** @brief Set of nonces */
typedef std::set<Nonce, key_compare> NonceSet;

/** @brief Set of nonce hashes */
typedef std::set<Hash> HashSet;

/** @brief Type for node identification/address */
typedef LAddress::L3Type Address;

/** @brief Type storing keys for all neighboring nodes */
typedef std::map<Address, Key> Keys;

/** @brief Type storing nonces for all neighboring nodes */
typedef std::map<Address,  std::map<Hash, Nonce> > Nonces;

/** @brief Set of hashes assigned to particular node */
typedef std::map<Address, HashSet> Hashes;

/** @brief Counter structure used in nonce lifecycle handling */
struct CountStruct{
    int counter;
    bool keyUpdated;
};

/** @brief Handling the whole nonce lifecycle */
typedef std::map<Address,  std::map<Hash, CountStruct> > Counters;

/** @brief Structure for common neighbours together with respective distances */
typedef std::map<Address, std::pair<double, double>> CommonNeighbourDist;

/** @brief Common neighbours for all node's neighbours  */
typedef std::map<Address, CommonNeighbourDist> CommonNeighbourDistAll;

/** @brief Set of nodes */
typedef std::set<Address> NodesSet;

#endif
