#include "utilfunctions.h"
#include "sha256.h"

bool areNodesNeighbours(cModule * nodeA, cModule * nodeB) {
    cModule::GateIterator * neighbourGates = new cModule::GateIterator(nodeA);

    for (; !neighbourGates->end(); (*neighbourGates)++) {
        cGate *gate = (*neighbourGates)();
        cGate *otherGate = gate->getType() == cGate::OUTPUT ? gate->getNextGate() : NULL;
        if (otherGate) {
            if (otherGate->getOwnerModule()->getIndex() == nodeB->getIndex()) {
                return true;
            }
        }
    }

    return false;
}

bool existsSharedKeyBetween(int nodeIndexA, Data * dataA, int nodeIndexB, Data * dataB) {
    int keyLengthA = dataA->getKeyLength();

    if (keyLengthA == dataB->getKeyLength()) {
        if (dataA->getKey(nodeIndexB) != NULL && dataB->getKey(nodeIndexA) != NULL)
            return memcmp(dataA->getKey(nodeIndexB), dataB->getKey(nodeIndexA), keyLengthA) == 0;
    }

    return false;
}

void rndArray(Key key, int length) {
    for (int i = 0; i < length; i++)
        key[i] = rand() & 255;
}


void keyToHex(Key key, int keyLength) {
    if (key != NULL) {

        const char * pszNibbleToHex = {"0123456789ABCDEF"};
        int hexStringSize = 4*keyLength+1;
        unsigned char * buffer = new unsigned char[hexStringSize];
        memset(buffer, 0, hexStringSize);

        for (int i = 0; i < keyLength; i++) {
            buffer[2*i]   = pszNibbleToHex[key[i] >> 4];
            buffer[2*i+1] = pszNibbleToHex[key[i] & 0xF];
        }

        EV << buffer << endl;

        delete [] buffer;
    }
}

unsigned int SHA256int( unsigned char * input, int inputLength){
    sha256_context sha256_ctx;
    unsigned char hash[32] = { 0 };
    memset(&sha256_ctx, 0, sizeof(sha256_context));
    sha256_starts(&sha256_ctx, 0);
    sha256_update(&sha256_ctx, input, inputLength);
    sha256_finish(&sha256_ctx, hash);

    return ((unsigned int*)hash)[0];
}

Key SHA256( unsigned char * input, int inputLength){
    sha256_context sha256_ctx;
    Key  hash = new unsigned char [32];
    memset(&sha256_ctx, 0, sizeof(sha256_context));
    sha256_starts(&sha256_ctx, 0);
    sha256_update(&sha256_ctx, input, inputLength);
    sha256_finish(&sha256_ctx, hash);

    return hash;
}
